#include "Audio.h"

static bool playing;
static AudioDeviceID device;
static UInt32 bufferSize;
static AudioStreamBasicDescription format;
static AudioCallback callback;

static OSStatus proc(AudioDeviceID, const AudioTimeStamp *, const AudioBufferList *, const AudioTimeStamp *, AudioBufferList *outOutputData, const AudioTimeStamp *, void *) {
	if (callback) callback((float *)outOutputData->mBuffers[0].mData, bufferSize / format.mBytesPerFrame);
}

void AudioSetup(AudioCallback func) {
	UInt32 count = sizeof(device);
	if (AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,  &count, (void *)&device) != kAudioHardwareNoError) return;
	count = sizeof(bufferSize);
	if (AudioDeviceGetProperty(device, 0, false, kAudioDevicePropertyBufferSize, &count, &bufferSize) != kAudioHardwareNoError) return;
	count = sizeof(format);
	if (AudioDeviceGetProperty(device, 0, false, kAudioDevicePropertyStreamFormat, &count, &format) != kAudioHardwareNoError) return;
	if (format.mFormatID != kAudioFormatLinearPCM) return;
	if (!(format.mFormatFlags & kLinearPCMFormatFlagIsFloat)) return;
	callback = func;
}

void AudioStart() {
	if (!callback || playing) return;
	if (AudioDeviceAddIOProc(device, proc, 0) != kAudioHardwareNoError) return;
	if (AudioDeviceStart(device, proc) != kAudioHardwareNoError) return;
	playing = true;
}

void AudioStop() {
	if (!callback || !playing) return;
	if (AudioDeviceStop(device, proc) != kAudioHardwareNoError) return;
	if (AudioDeviceRemoveIOProc(device, proc) != kAudioHardwareNoError) return;
	playing = false;
}

