#import "Audio.h"

class Voice {
public:
	enum { N = 8 };
	enum { FS = 44100 };
	Voice() { Init(); }
	static void NoteOn(int num, int note, int velo) {
		voice[num].On(note, velo);
	}
	static void NoteOff(int num) {
		voice[num].Init();
	}
	static void MakeWave(int n) {
		while (n-- > 0) {
			int v = 0;
			for (Voice *p = voice; p < voice + N; p++) 
				v += p->Step();
			*destp++ = 0.004 * v;
			*destp++ = 0.004 * v;
		}
	}
	static void Silent(int num) {
		while (num--) *destp++ = 0.0, *destp++ = 0.0;
	}
	static void SetDestBuf(float *buf) { destp = buf; }
private:
	void Init() {
		preset = count = value = 0;
	}
	void On(int note, int velo) {
		count = preset = int(FS / (440.0 * pow(2.0, (note - 69) / 12.0)) + 0.5) >> 1;
		value = velo;
	}
	int Step() {
		if (!--count) {
			count = preset;
			value = -value;
		}
		return value;
	}
	int preset, count, value;
	static Voice voice[N];
	static float *destp;
	static int offcount;
};

Voice Voice::voice[Voice::N];
float *Voice::destp;
int Voice::offcount;

static UInt16 *buf, *bufp;
static long len;

static void generate(float *buffer, int numSamples) {
	Voice::SetDestBuf(buffer);
	while (numSamples > 0) {
		if (bufp < buf + len / 2) {
			if (*bufp & 0x8000) {
				int n = 0x10000 - *bufp;
				if (n < numSamples) {
					Voice::MakeWave(n);
					numSamples -= n;
					bufp++;
				}
				else {
					Voice::MakeWave(numSamples);
					*bufp += numSamples;
					numSamples = 0;
				}
			}
			else if (*bufp & 0x7f) {
				Voice::NoteOn(*bufp >> 11, *bufp & 0x7f, *bufp >> 7 & 15);
				bufp++;
			}
			else {
				Voice::NoteOff(*bufp >> 11);
				bufp++;
			}
		}
		else {
			Voice::Silent(numSamples);
			numSamples = 0;
		}
	}
}

@interface OrgelController : NSObject
{
}
@end

@implementation OrgelController

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	NSFileHandle *h = [NSFileHandle fileHandleForReadingAtPath:@"../../mid2pic/orgel.dat"];
	NSData *d = [h readDataToEndOfFile];
	len = [d length];
	buf = bufp = (UInt16 *)malloc(len);
	[d getBytes:buf length:len];
    AudioSetup(generate);
    AudioStart();
}

- (void)applicationWillTerminate:(NSNotification *)aNotification {
	AudioStop();
	free(buf);
}

@end

